"use client";

import * as React from "react";
import { useRouter, useSearchParams } from "next/navigation";
import { Loader2, MessageCircleCode } from "lucide-react";

import { cn } from "@/lib/utils";
import { useToast } from "@/hooks/use-toast";
import { buttonVariants } from "@/components/ui/button";

interface DiscourseData {
  sso_url: string;
}

export function UserAuthForm({
  className,
  ...props
}: React.HTMLAttributes<HTMLDivElement>) {
  const [isLoading, setIsLoading] = React.useState<boolean>(false);
  const router = useRouter();
  const { toast } = useToast();
  const searchParams = useSearchParams();
  const formRef = React.useRef<HTMLFormElement>(null);

  async function signIn() {
    try {
      const body: Record<string, any> = {};
      const callbackUrl = searchParams?.get("callbackUrl");

      console.log("Original callbackUrl:", callbackUrl);

      // 如果是 OAuth 回调，则保留完整的 OAuth 参数
      if (callbackUrl?.includes("/oauth/authorize")) {
        try {
          // 先解码一次 URL
          const decodedUrl = decodeURIComponent(callbackUrl);
          console.log("Decoded URL:", decodedUrl);

          // 解析 URL 并保留所有参数
          const url = new URL(decodedUrl);
          if (url.pathname === "/oauth/authorize") {
            body.return_url = `${window.location.origin}${url.pathname}${url.search}`;
            console.log("Extracted return_url:", body.return_url);
          }
        } catch (e) {
          console.error("URL 处理错误:", e);
          body.return_url = `${window.location.origin}/authorize`;
        }
      }

      console.log("发送请求体:", body);
      const response = await fetch("/api/auth/q58", {
        method: "POST",
        headers: {
          "Content-Type": "application/json",
        },
        body: JSON.stringify(body),
      });

      if (!response.ok) {
        const errorText = await response.text();
        console.error("服务器响应:", errorText);
        throw new Error(`登录请求失败: ${errorText}`);
      }

      const data = await response.json();
      console.log("收到 SSO URL:", data.sso_url);

      // 解析 SSO URL 并提取参数
      const ssoUrl = new URL(data.sso_url);
      const sso = ssoUrl.searchParams.get("sso");
      const sig = ssoUrl.searchParams.get("sig");

      // 使用表单提交方式跳转
      if (formRef.current && sso && sig) {
        const form = formRef.current;
        // 创建隐藏的输入字段
        const ssoInput = document.createElement("input");
        ssoInput.type = "hidden";
        ssoInput.name = "sso";
        ssoInput.value = sso;
        form.appendChild(ssoInput);

        const sigInput = document.createElement("input");
        sigInput.type = "hidden";
        sigInput.name = "sig";
        sigInput.value = sig;
        form.appendChild(sigInput);

        // 设置表单目标 URL（不带参数）
        form.action = `${ssoUrl.origin}${ssoUrl.pathname}`;
        form.submit();
      }
    } catch (error) {
      console.error("登录错误:", error);
      toast({
        title: "错误",
        description:
          error instanceof Error
            ? error.message
            : "登录过程中发生错误，请稍后重试",
        variant: "destructive",
      });
      setIsLoading(false);
    }
  }

  return (
    <div className={cn("grid gap-3", className)} {...props}>
      <form
        ref={formRef}
        method="get"
        target="_self"
        style={{ display: "none" }}
      />
      <button
        type="button"
        className={cn(buttonVariants({ variant: "outline" }))}
        onClick={() => {
          setIsLoading(true);
          signIn();
        }}
        disabled={isLoading}
      >
        {isLoading ? (
          <Loader2 className="mr-2 size-4 animate-spin" />
        ) : (
          <MessageCircleCode className="mr-2 size-4" />
        )}{" "}
        Q58论坛
      </button>
    </div>
  );
}
